@@ -1,26 +1,31 @@
+#!/usr/bin/env python
 # Copyright: (c) 2017, Ansible Project
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
+# PYTHON_ARGCOMPLETE_OK
 
 from __future__ import (absolute_import, division, print_function)
 __metaclass__ = type
 
+# ansible.cli needs to be imported first, to ensure the source bin/* scripts run that code first
+from ansible.cli import CLI
+
 import os
 import shlex
 import subprocess
+
+from collections.abc import Mapping
+
 import yaml
 
 from ansible import context
 import ansible.plugins.loader as plugin_loader
 
 from ansible import constants as C
-from ansible.cli import CLI
 from ansible.cli.arguments import option_helpers as opt_help
 from ansible.config.manager import ConfigManager, Setting
 from ansible.errors import AnsibleError, AnsibleOptionsError
 from ansible.module_utils._text import to_native, to_text, to_bytes
-from ansible.module_utils.common._collections_compat import Mapping
 from ansible.module_utils.six import string_types
-from ansible.module_utils.six.moves import shlex_quote
 from ansible.parsing.quoting import is_quoted
 from ansible.parsing.yaml.dumper import AnsibleDumper
 from ansible.utils.color import stringc
@@ -33,6 +38,8 @@ display = Display()
 class ConfigCLI(CLI):
     """ Config command line class """
 
+    name = 'ansible-config'
+
     def __init__(self, args, callback=None):
 
         self.config_file = None
@@ -257,7 +264,7 @@ class ConfigCLI(CLI):
                                 # list of other stuff
                                 default = '%s' % to_native(default)
                     if isinstance(default, string_types) and not is_quoted(default):
-                        default = shlex_quote(default)
+                        default = shlex.quote(default)
                 elif default is None:
                     default = ''
 
@@ -365,18 +372,23 @@ class ConfigCLI(CLI):
 
         text = []
         for setting in sorted(config):
+            changed = False
             if isinstance(config[setting], Setting):
+                # proceed normally
                 if config[setting].origin == 'default':
                     color = 'green'
                 elif config[setting].origin == 'REQUIRED':
+                    # should include '_terms', '_input', etc
                     color = 'red'
                 else:
                     color = 'yellow'
+                    changed = True
                 msg = "%s(%s) = %s" % (setting, config[setting].origin, config[setting].value)
             else:
                 color = 'green'
                 msg = "%s(%s) = %s" % (setting, 'default', config[setting].get('default'))
-            if not context.CLIARGS['only_changed'] or color == 'yellow':
+
+            if not context.CLIARGS['only_changed'] or changed:
                 text.append(stringc(msg, color))
 
         return text
@@ -440,6 +452,11 @@ class ConfigCLI(CLI):
                         o = 'REQUIRED'
                     else:
                         raise e
+
+                if v is None and o is None:
+                    # not all cases will be error
+                    o = 'REQUIRED'
+
                 config_entries[finalname][setting] = Setting(setting, v, o, None)
 
             # pretty please!
@@ -462,10 +479,20 @@ class ConfigCLI(CLI):
             text = self._get_global_configs()
             # deal with plugins
             for ptype in C.CONFIGURABLE_PLUGINS:
-                text.append('\n%s:\n%s' % (ptype.upper(), '=' * len(ptype)))
-                text.extend(self._get_plugin_configs(ptype, context.CLIARGS['args']))
+                plugin_list = self._get_plugin_configs(ptype, context.CLIARGS['args'])
+                if not context.CLIARGS['only_changed'] or plugin_list:
+                    text.append('\n%s:\n%s' % (ptype.upper(), '=' * len(ptype)))
+                    text.extend(plugin_list)
         else:
             # deal with plugins
             text = self._get_plugin_configs(context.CLIARGS['type'], context.CLIARGS['args'])
 
         self.pager(to_text('\n'.join(text), errors='surrogate_or_strict'))
+
+
+def main(args=None):
+    ConfigCLI.cli_executor(args)
+
+
+if __name__ == '__main__':
+    main()
